/*
 * Pan - A Newsreader for X
 * Copyright (c) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "pan.h"

gint one_click_preview = 0;

/*---[ gui_construct_paned ]------------------------------------------
 * build the paned GUI layout
 *--------------------------------------------------------------------*/
void
gui_paned_construct (void)
{
	GtkWidget *vpane;
	GtkWidget *hpane;

	if (viewmode == GUI_PANED) return;
	if (viewmode == GUI_NOTEBOOK)
	{
		gtk_widget_ref (groups_vbox);
		gtk_container_remove (GTK_CONTAINER (groups_vbox->parent), groups_vbox);
		gtk_widget_ref (articlelist_ctree);
		gtk_container_remove (GTK_CONTAINER (articlelist_ctree->parent), articlelist_ctree);
		gtk_widget_ref (text_box);
		gtk_container_remove (GTK_CONTAINER (text_box->parent), text_box);
		gtk_container_foreach (GTK_CONTAINER (contents_vbox), (GtkCallback) gtk_widget_destroy, NULL);
	}

/* saving old layout 
	hpane = gtk_hpaned_new ();
	gtk_paned_add1 (GTK_PANED(hpane), groups_vbox);
	gtk_paned_add2 (GTK_PANED(hpane), articlelist_ctree);

	vpane = gtk_vpaned_new ();
	gtk_paned_add1 (GTK_PANED(vpane), hpane);
	gtk_paned_add2 (GTK_PANED(vpane), text_box);
*/

	vpane = gtk_vpaned_new ();
	gtk_paned_add1 (GTK_PANED (vpane), articlelist_ctree);
	gtk_paned_add2 (GTK_PANED (vpane), text_box);
	gtk_widget_set_usize (articlelist_ctree, -1, 200);

	hpane = gtk_hpaned_new ();
	gtk_paned_add1 (GTK_PANED (hpane), groups_vbox);
	gtk_paned_add2 (GTK_PANED (hpane), vpane);

	gtk_box_pack_start (GTK_BOX (contents_vbox), hpane, TRUE, TRUE, 0);

//	infobar = gui_create_infobar ();
//	gtk_box_pack_end (GTK_BOX (contents_vbox), infobar, FALSE, FALSE, 0);

/* connect signals */
	gtk_signal_connect (GTK_OBJECT (Pan.article_ctree), "key_press_event",
			    GTK_SIGNAL_FUNC (gui_key_press_cb), NULL);
	gtk_signal_connect (GTK_OBJECT (Pan.group_clist), "key_press_event",
			    GTK_SIGNAL_FUNC (gui_key_press_cb),  NULL);
	gtk_signal_connect (GTK_OBJECT (text), "key_press_event",
			    GTK_SIGNAL_FUNC (gui_key_press_cb), NULL);


	gui_restore_column_widths (GTK_CLIST (Pan.group_clist), "group");
	gui_restore_column_widths (GTK_CLIST (Pan.article_ctree), "articlelist");

	if (viewmode == GUI_NOTEBOOK)
	{
		gtk_widget_unref (groups_vbox);
		gtk_widget_unref (articlelist_ctree);
		gtk_widget_unref (text_box);
	}

	viewmode = GUI_PANED;
	gtk_widget_show_all (GTK_WIDGET(contents_vbox));
}


/*--------------------------------------------------------------------
 * get the current pane, returns an integer reflecting which pane
 * the user is 'in' ... almost identical to the gtk_notebook_get_page
 * functions
 *--------------------------------------------------------------------*/
int
gui_paned_get_current_pane (void)
{
	if (GTK_WIDGET_HAS_FOCUS (Pan.group_clist))
		return 0;
	if (GTK_WIDGET_HAS_FOCUS (Pan.article_ctree))
		return 1;
	if (GTK_WIDGET_HAS_FOCUS (text))
		return 2;
	
	return -1;
}


/*--------------------------------------------------------------------
 * ENTER key pressed while in the paned mode, process accordingly
 *--------------------------------------------------------------------*/
void
gui_paned_enter_clicked (void)
{
	group_data *gdata;
	article_data *adata;

	switch (gui_paned_get_current_pane()) {
	case 0: /* enter was pressed inside the 'group list' pane */
		gdata = (group_data *) gtk_clist_get_row_data (GTK_CLIST (Pan.group_clist), 
							       GPOINTER_TO_INT (GTK_CLIST (Pan.group_clist)->selection->data));
		if (gdata) {
			if (Pan.active_group == gdata) {
				gtk_widget_grab_focus (Pan.article_ctree);
				break;
			}
			if (group_index_exists(gdata))
				queue_add_articlelist_download (gdata,
								QUEUE_ARTICLELIST_LOAD |
								QUEUE_FOREGROUND);
			else
				choose_download_dialog (gdata);
		}
		gui_set_title (gdata);
		break;
	case 1: /* enter key pressed inside the 'article list' pane */
		adata = (article_data *) gtk_ctree_node_get_row_data (GTK_CTREE (Pan.article_ctree), 
								      GTK_CLIST (Pan.article_ctree)->selection->data);
		if (adata) {
			current_article = adata;
			if (adata->body == NULL) {
				queue_add_article_download (adata,
							    QUEUE_ARTICLE_DOWNLOAD |
							    QUEUE_FOREGROUND);
				adata->state |= STATE_DOWNLOAD_QUEUED;
//				articlelist_update_node (adata);
			}
			else
				text_update (adata);
		}
		break;
	case 2: /* enter key pressed inside the 'message' pane */
		break;
	default:
		g_print ("reached default case in gui_paned_enter_clicked()");
		break;
	}
}
