/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Matt Eagleson (e.Messiah@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#include "pan.h"

GtkWidget *text;
GtkWidget *text_rightclick_popup;

article_data *article;

static GnomeUIInfo text_menu_popup[] = {
	{ GNOME_APP_UI_ITEM, N_("Reply to Sender"), N_("Create a reply to the sender."), followup_window },
	{ GNOME_APP_UI_ITEM, N_("Forward"), N_("Forward this message to someone."), not_implemented },
	GNOMEUIINFO_SEPARATOR,
	{ GNOME_APP_UI_ITEM, N_("Select All"), N_("Select all messages"), text_select_all,
	  NULL, NULL, GNOME_APP_PIXMAP_NONE, NULL, 'A', GDK_CONTROL_MASK, NULL},
	GNOMEUIINFO_END
};


/* FIXME: no code, no functionality */
void
text_select_all (GtkWidget *widget, gpointer data)
{
	gtk_editable_select_region (GTK_EDITABLE (text), 0, -1);
}


/*--------------------------------------------------------------------
 * called when the 'realize' signal is sent to the text widget,
 * not sure what it really does though... add comments ???
 *--------------------------------------------------------------------*/
void
realize_text_cb (GtkWidget *text, gpointer data)
{
	text_clear ();
	gtk_text_insert (GTK_TEXT(text), NULL, &text->style->black, NULL, "", -1);
}


/*---[ text_clear ]---------------------------------------------------
 * wipe the text widget clean
 *--------------------------------------------------------------------*/
void
text_clear (void)
{
	gint i;

	pan_lock();
	i = gtk_text_get_length (GTK_TEXT(text));
	if (i > 0) {
		gtk_text_set_point (GTK_TEXT(text), i);
		gtk_text_backward_delete (GTK_TEXT(text), i);
	}
	gtk_widget_queue_draw (GTK_WIDGET (text));
	pan_unlock ();
}


/*--------------------------------------------------------------------
 * 
 *--------------------------------------------------------------------*/
void
text_button_press (GtkWidget *widget, GdkEventButton *bevent)
{
	gtk_widget_grab_focus (GTK_WIDGET (text));

	if (bevent->button != 1)
		gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "button_press_event");
	
	if (bevent->button == 3)
		gtk_menu_popup (GTK_MENU(text_rightclick_popup), NULL, NULL, NULL,
				NULL, bevent->button, bevent->time);
}


/*---[ text_create ]--------------------------------------------------
 * create a scrolled text area that displays a message's body
 *--------------------------------------------------------------------*/
GtkWidget *
text_create (void)
{
	GtkWidget *scrolled_window;

	text = gtk_text_new (NULL, NULL);
	gtk_text_set_editable (GTK_TEXT(text), FALSE);
	gtk_text_set_word_wrap (GTK_TEXT(text), TRUE);

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
					GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	/* Add a handler to put a message in the text widget when it is realized */
	gtk_signal_connect (GTK_OBJECT(text), "realize",
			    GTK_SIGNAL_FUNC (realize_text_cb), NULL);

	gtk_signal_connect (GTK_OBJECT(text), "button_press_event",
			    GTK_SIGNAL_FUNC (text_button_press), NULL);

	gtk_container_add (GTK_CONTAINER (scrolled_window), text);

	text_rightclick_popup = gnome_popup_menu_new (text_menu_popup);

	return scrolled_window;

}


article_data *
displayed_article (void)
{
	return article;
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
void
text_update (article_data *adata)
{
	int headers;

	if (adata == NULL)
		return;

	headers = gnome_config_get_int ("/Pan/Message/Headers=1");

	text_clear ();

	pan_lock ();
	acache_load (adata);

	if (article == NULL)
		article = g_malloc0 (sizeof (article_data));
	article_free (article);
	article = article_copy (adata);

	gtk_text_freeze (GTK_TEXT(text));

	switch (headers) {
	case 0:
		/* Brief */
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, "\"", -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, adata->subject, -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, _("\" from "), -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, adata->author, -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, "\n--\n\n", -1);
		break;
	case 1:
		/* Partial */
		gtk_label_set_text (GTK_LABEL (from_label), adata->author);
		gtk_label_set_text (GTK_LABEL (subject_label), adata->subject);

/*
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, "Subject: ", -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, adata->subject, -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, "\n", -1);
		
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, "Author: ", -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, adata->author, -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, "\n", -1);
		
		p = ctime((time_t *) &adata->date);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, "Date: ", -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, p, -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, "--\n\n", -1);
*/
		break;
	case 2:
		/* Full */	
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, adata->header, -1);
		gtk_text_insert(GTK_TEXT(text), NULL, &text->style->black, NULL, "--\n\n", -1);
		break;
	default:
		break;
	}
	gtk_text_insert (GTK_TEXT(text), NULL, &text->style->black, NULL, adata->body, -1);
	gtk_text_thaw (GTK_TEXT(text));
	gtk_widget_queue_draw (GTK_WIDGET (text));
	pan_unlock ();
	g_free (adata->header);
	adata->header = NULL;
	g_free (adata->body);
	adata->body = NULL;
}
